package com.qmth.sop.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.bean.dto.*;
import com.qmth.sop.business.bean.result.CrmProjectResult;
import com.qmth.sop.business.bean.result.UserArchivesResult;
import com.qmth.sop.business.entity.*;
import com.qmth.sop.business.mapper.TBDingStatisticMapper;
import com.qmth.sop.business.service.*;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.DingDateTypeEnum;
import com.qmth.sop.common.enums.ExceptionResultEnum;
import com.qmth.sop.common.enums.SopRoleTypeEnum;
import com.qmth.sop.common.util.DateDisposeUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * 考勤提交统计表 服务实现类
 * </p>
 *
 * @author CaoZixuan
 * @since 2024-05-22
 */
@Service
public class TBDingStatisticServiceImpl extends ServiceImpl<TBDingStatisticMapper, TBDingStatistic> implements TBDingStatisticService {

    private final static Logger log = LoggerFactory.getLogger(TBDingStatisticServiceImpl.class);

    @Resource
    private TBCrmDetailService tbCrmDetailService;

    @Resource
    private TBDingService tbDingService;

    @Resource
    private TBCrmService tbCrmService;

    @Resource
    private TBUserArchivesService tbUserArchivesService;

    @Resource
    private TBDingApplyService tbDingApplyService;

    @Resource
    private TBDingSubmitService tbDingSubmitService;

    @Resource
    private TBServiceService tbServiceService;

    @Resource
    private SysCustomService sysCustomService;

    @Resource
    private SysSupplierService sysSupplierService;

    @Override
    public TBDingStatistic findBySopNoAndUserArchivesId(Long serviceId, String sopNo, Long userArchivesId) {
        QueryWrapper<TBDingStatistic> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(TBDingStatistic::getServiceId, serviceId).eq(TBDingStatistic::getUserArchivesId, userArchivesId).last(SystemConstant.LIMIT1);
        if (SystemConstant.strNotNull(sopNo)) {
            queryWrapper.lambda().eq(TBDingStatistic::getSopNo, sopNo);
        } else {
            queryWrapper.lambda().isNull(TBDingStatistic::getSopNo);
        }
        return this.getOne(queryWrapper);
    }

    @Override
    public void buildDingStatistic(String sopNo, Long userArchivesId) {
        Long currentEarliestTime = DateDisposeUtils.getEarliestTime(System.currentTimeMillis());
        List<TBDing> datasource = tbDingService.list(new QueryWrapper<TBDing>().lambda().eq(TBDing::getSopNo, sopNo).eq(TBDing::getUserArchivesId, userArchivesId)).stream().filter(e -> {
            if (Objects.nonNull(e.getSignInTime()) && Objects.nonNull(e.getSignOutTime())) {
                return e.getSignInTime() < currentEarliestTime && e.getSignOutTime() < currentEarliestTime;
            } else if (Objects.isNull(e.getSignInTime())) {
                return e.getSignOutTime() < currentEarliestTime;
            } else if (Objects.isNull(e.getSignOutTime())) {
                return e.getSignInTime() < currentEarliestTime;
            } else {
                return true;
            }
        }).collect(Collectors.toList());
        this.buildDingStatisticBySop(sopNo, datasource, -1L);
    }

    @Transactional
    @Override
    public void buildDingStatistic(String sopNo) {
        // 统计前一天的数据
        Long currentEarliestTime = DateDisposeUtils.getEarliestTime(System.currentTimeMillis());
        List<TBDing> datasource = tbDingService.list(new QueryWrapper<TBDing>().lambda().eq(TBDing::getSopNo, sopNo)).stream().filter(e -> {
            if (Objects.nonNull(e.getSignInTime()) && Objects.nonNull(e.getSignOutTime())) {
                return e.getSignInTime() < currentEarliestTime && e.getSignOutTime() < currentEarliestTime;
            } else if (Objects.isNull(e.getSignInTime())) {
                return e.getSignOutTime() < currentEarliestTime;
            } else if (Objects.isNull(e.getSignOutTime())) {
                return e.getSignInTime() < currentEarliestTime;
            } else {
                return true;
            }
        }).collect(Collectors.toList());
        this.buildDingStatisticBySop(sopNo, datasource, -1L);
    }

    @Transactional
    @Override
    public void buildDingStatistic() {
        Long currentEarliestTime = DateDisposeUtils.getEarliestTime(System.currentTimeMillis());

        // 统计前一天的数据
        List<TBDing> datasource = tbDingService.list().stream().filter(e -> {
            if (Objects.nonNull(e.getSignInTime()) && Objects.nonNull(e.getSignOutTime())) {
                return e.getSignInTime() < currentEarliestTime && e.getSignOutTime() < currentEarliestTime;
            } else if (Objects.isNull(e.getSignInTime())) {
                return e.getSignOutTime() < currentEarliestTime;
            } else if (Objects.isNull(e.getSignOutTime())) {
                return e.getSignInTime() < currentEarliestTime;
            } else {
                return true;
            }
        }).collect(Collectors.toList());
        // 工程师考勤统计（sopNo不为空的 按照sopNo统计）
        List<String> sopNoList = datasource.stream().map(TBDing::getSopNo).filter(SystemConstant::strNotNull).distinct()
                .collect(Collectors.toList());
        for (String sopNo : sopNoList) {
            List<TBDing> sopDingInfo = datasource.stream().filter(e -> SystemConstant.strNotNull(e.getSopNo()) && e.getSopNo().equals(sopNo))
                    .collect(Collectors.toList());
            this.buildDingStatisticBySop(sopNo, sopDingInfo, -1L);
        }

        // 区协考勤统计 （sopNo为空的 按照服务单元统计）
        List<Long> serviceIdList = datasource.stream().filter(e -> !SystemConstant.strNotNull(e.getSopNo())).map(TBDing::getServiceId).distinct().collect(Collectors.toList());
        for (Long serviceId : serviceIdList) {
            List<TBDing> serviceDingInfo = datasource.stream().filter(e -> e.getServiceId().equals(serviceId) && !SystemConstant.strNotNull(e.getSopNo()))
                    .collect(Collectors.toList());
            this.buildDingStatisticByService(serviceId, serviceDingInfo, -1L);
        }
    }

    @Transactional
    @Override
    public void buildDingStatisticBySop(String sopNo, List<TBDing> datasource, Long requestUserId) {
        List<TBDingStatistic> tbDingStatisticList = new ArrayList<>();
        TBCrmDetail tbCrmDetail = tbCrmDetailService.findBySopNo(sopNo);
        if (Objects.isNull(tbCrmDetail)) {
            log.error(String.format("sopNo[%s]未找到派单详情信息", sopNo));
            throw ExceptionResultEnum.ERROR.exception("未找到派单详情信息");
        }
        Long scanActualStartTime = tbCrmDetail.getScanActualStartTime();
        Long scanActualEndTime = tbCrmDetail.getScanActualEndTime();
        Long markPaperActualStartTime = tbCrmDetail.getMarkPaperActualStartTime();
        Long markPaperActualEndTime = tbCrmDetail.getMarkPaperActualEndTime();
        if (Objects.nonNull(scanActualStartTime)) {
            scanActualStartTime = DateDisposeUtils.getEarliestTime(scanActualStartTime);
        }
        if (Objects.nonNull(scanActualEndTime)) {
            scanActualEndTime = DateDisposeUtils.getLatestTime(scanActualEndTime);
        }
        if (Objects.nonNull(markPaperActualStartTime)) {
            markPaperActualStartTime = DateDisposeUtils.getEarliestTime(markPaperActualStartTime);
        }
        if (Objects.nonNull(markPaperActualEndTime)) {
            markPaperActualEndTime = DateDisposeUtils.getLatestTime(markPaperActualEndTime);
        }

        String crmNo = tbCrmDetail.getCrmNo();
        TBCrm tbCrm = tbCrmService.findByCrmNo(crmNo);
        if (Objects.isNull(tbCrm)) {
            log.error(String.format("crmNo[%s]未找到派单信息", crmNo));
            throw ExceptionResultEnum.ERROR.exception("未找到派单信息");
        }
        CrmProjectResult crmProjectResult = tbCrmService.findCrmProjectInfo(null, null, crmNo);
        Long serviceUnitId = tbCrm.getServiceId();

        // 该sop计划有效的日期
        List<Long> userArchivesIdList = datasource.stream().map(TBDing::getUserArchivesId).distinct().collect(Collectors.toList());

        for (Long userArchivesId : userArchivesIdList) {
            UserArchivesResult tbUserArchives = tbUserArchivesService.findUserArchivesByArchivesIdORUserId(
                    userArchivesId, null);
            if (Objects.isNull(tbUserArchives)) {
                throw ExceptionResultEnum.ERROR.exception("人员档案不存在");
            }
            Long userId = tbUserArchives.getUserId();
            // 打卡数据集合
            List<TBDing> dingList = datasource.stream()
                    .filter(e -> Objects.equals(userArchivesId, e.getUserArchivesId())).collect(Collectors.toList());

            // 非异常打卡（打卡时间在计划时间范围内 以及签到签退完整的）
            Long finalScanActualStartTime = scanActualStartTime;
            Long finalScanActualEndTime = scanActualEndTime;
            Long finalMarkPaperActualStartTime = markPaperActualStartTime;
            Long finalMarkPaperActualEndTime = markPaperActualEndTime;
            List<TBDing> effectDingList = dingList.stream()
                    .filter(e -> !SystemConstant.isOneNull(e.getSignInTime(), e.getSignOutTime())).filter(e -> {
                        // 在区协打卡时间范围内
                        Long signInTime = e.getSignInTime();
                        Long signOutTime = e.getSignOutTime();
                        boolean inScan = false;
                        boolean inMarkPaper = false;
                        if (!SystemConstant.isOneNull(finalScanActualStartTime, finalScanActualEndTime)) {
                            inScan = signInTime >= finalScanActualStartTime && signInTime <= finalScanActualEndTime
                                    && signOutTime >= finalScanActualStartTime && signOutTime <= finalScanActualEndTime;
                        }
                        if (!SystemConstant.isOneNull(finalMarkPaperActualStartTime, finalMarkPaperActualEndTime)) {
                            inMarkPaper = signInTime >= finalMarkPaperActualStartTime && signInTime <= finalMarkPaperActualEndTime
                                    && signOutTime >= finalMarkPaperActualStartTime && signOutTime <= finalMarkPaperActualEndTime;
                        }

                        return inScan || inMarkPaper;
                    }).collect(Collectors.toList());

            List<SopRoleTypeEnum> sopRoleTypeList = dingList.stream().flatMap(e -> {
                String sopRoleTypeStr = e.getSopRoleType();
                List<SopRoleTypeEnum> list = JSONArray.parseArray(sopRoleTypeStr, SopRoleTypeEnum.class);
                return list.stream();
            }).distinct().collect(Collectors.toList());
            String sopRoleType = JSON.toJSONString(sopRoleTypeList);

            // 根据非异常打卡数据统计
            int actualDays = 0;
            int weekDays = 0;
            int weekends = 0;
            int legalHolidays = 0;
            BigDecimal workHours = BigDecimal.ZERO;
            BigDecimal var = new BigDecimal(3600000);
            for (TBDing effectDing : effectDingList) {
                if (!SystemConstant.isOneNull(effectDing.getSignInTime(), effectDing.getSignOutTime())) {
                    // 有效天数
                    actualDays++;
                    DingDateTypeEnum dateType = effectDing.getDateType();
                    switch (dateType) {
                    case WEEKEND:
                        weekends++;
                        break;
                    case LEGAL_HOLIDAYS:
                        legalHolidays++;
                        break;
                    default:
                        weekDays++;
                        break;
                    }
                    // 累计工时
                    workHours = workHours.add(
                            new BigDecimal(effectDing.getSignOutTime() - effectDing.getSignInTime()).divide(var, 1,
                                    RoundingMode.HALF_UP));
                }
            }
            // 违规工时
            int violationDays = Math.toIntExact(
                    dingList.stream().filter(e -> SystemConstant.isOneNull(e.getSignInTime(), e.getSignOutTime()))
                            .count());
            // 考勤异常数
            int dingExceptionCount = 0;
            for (TBDing tbDing : dingList) {
                Long signInTime = tbDing.getSignInTime();
                Long signOutTime = tbDing.getSignOutTime();
                if (!SystemConstant.longNotNull(signInTime)) {
                    dingExceptionCount++;
                }
                if (!SystemConstant.longNotNull(signOutTime)) {
                    dingExceptionCount++;
                }
            }

            // 剩余补卡数
            int remainCount = tbDingService.findRemainCount(userId, serviceUnitId, sopNo);

            // 待处理异常数
            int exceptionCount = tbDingApplyService.count(
                    new QueryWrapper<TBDingApply>().lambda().eq(TBDingApply::getSopNo, sopNo).isNull(TBDingApply::getApprove));

            TBDingStatistic tbDingStatistic;
            tbDingStatistic = this.findBySopNoAndUserArchivesId(serviceUnitId, sopNo, userArchivesId);
            if (Objects.isNull(tbDingStatistic)) {
                // 新增考勤统计
                tbDingStatistic = new TBDingStatistic();
                tbDingStatistic.insertInfo(requestUserId);
            } else {
                // 更新考勤统计
                if (!tbDingSubmitService.canUpdateDingSetting(serviceUnitId, sopNo, userId)) {
                    // 已提交的考勤不再计算了
                    continue;
                }
                tbDingStatistic.updateInfo(requestUserId);
            }
            tbDingStatistic.setServiceId(serviceUnitId);
            tbDingStatistic.setServiceName(crmProjectResult.getServiceUnitName());
            tbDingStatistic.setSopNo(sopNo);
            tbDingStatistic.setCrmNo(crmNo);
            tbDingStatistic.setCustomId(crmProjectResult.getCustomId());
            tbDingStatistic.setCustomName(crmProjectResult.getCustomName());
            tbDingStatistic.setProvince(crmProjectResult.getCustomProvince());
            tbDingStatistic.setCity(crmProjectResult.getCustomCity());
            tbDingStatistic.setUserId(userId);
            tbDingStatistic.setUserArchivesId(userArchivesId);
            tbDingStatistic.setUserArchivesName(tbUserArchives.getName());
            tbDingStatistic.setUserArchivesCode(tbUserArchives.getCode());
            tbDingStatistic.setScanStartTime(tbCrmDetail.getScanActualStartTime());
            tbDingStatistic.setScanEndTime(tbCrmDetail.getScanActualEndTime());
            tbDingStatistic.setMarkPaperStartTime(tbCrmDetail.getMarkPaperActualStartTime());
            tbDingStatistic.setMarkPaperEndTime(tbCrmDetail.getMarkPaperActualEndTime());
            tbDingStatistic.setRoleName(sopRoleType);
            tbDingStatistic.setSupplierId(tbUserArchives.getSupplierId());
            tbDingStatistic.setSupplierName(tbUserArchives.getSupplierName());
            tbDingStatistic.setActualDays(actualDays);
            tbDingStatistic.setWeekdays(weekDays);
            tbDingStatistic.setWeekends(weekends);
            tbDingStatistic.setLegalHolidays(legalHolidays);
            tbDingStatistic.setWorkHours(workHours);
            tbDingStatistic.setViolationDays(violationDays);
            tbDingStatistic.setDingExceptionCount(dingExceptionCount);
            tbDingStatistic.setRemainCount(remainCount);
            tbDingStatistic.setExceptionCount(exceptionCount);
            tbDingStatisticList.add(tbDingStatistic);
        }
        this.saveOrUpdateBatch(tbDingStatisticList);
    }

    @Override
    public void buildDingStatisticByService(Long serviceId, List<TBDing> datasource, Long requestUserId) {
        RegionCoordinatorDingTimeDto dingTimeDto = tbDingService.findRegionCoordinatorDingTime();
        Long startTime = dingTimeDto.getStartTime();
        Long endTime = dingTimeDto.getEndTime();

        List<TBDingStatistic> tbDingStatisticList = new ArrayList<>();
        List<Long> userArchivesIdList = datasource.stream().map(TBDing::getUserArchivesId).distinct().collect(Collectors.toList());

        for (Long userArchivesId : userArchivesIdList) {
            UserArchivesResult tbUserArchives = tbUserArchivesService.findUserArchivesByArchivesIdORUserId(
                    userArchivesId, null);
            if (Objects.isNull(tbUserArchives)) {
                throw ExceptionResultEnum.ERROR.exception("人员档案不存在");
            }
            Long userId = tbUserArchives.getUserId();
            // 打卡数据集合
            List<TBDing> dingList = datasource.stream()
                    .filter(e -> Objects.equals(userArchivesId, e.getUserArchivesId())).collect(Collectors.toList());

            // 非异常打卡（打卡时间在计划时间范围内 以及签到签退完整的）
            List<TBDing> effectDingList = dingList.stream()
                    .filter(e -> !SystemConstant.isOneNull(e.getSignInTime(), e.getSignOutTime())).filter(e -> {
                        // 在区协打卡时间范围内
                        Long signInTime = e.getSignInTime();
                        Long signOutTime = e.getSignOutTime();
                        return signInTime > startTime && signInTime < endTime && signOutTime > startTime && signOutTime < endTime;
                    }).collect(Collectors.toList());

            List<SopRoleTypeEnum> sopRoleTypeList = dingList.stream().flatMap(e -> {
                String sopRoleTypeStr = e.getSopRoleType();
                List<SopRoleTypeEnum> list = JSONArray.parseArray(sopRoleTypeStr, SopRoleTypeEnum.class);
                return list.stream();
            }).distinct().collect(Collectors.toList());
            String sopRoleType = JSON.toJSONString(sopRoleTypeList);

            // 根据非异常打卡数据统计
            int actualDays = 0;
            int weekDays = 0;
            int weekends = 0;
            int legalHolidays = 0;
            BigDecimal workHours = BigDecimal.ZERO;
            BigDecimal var = new BigDecimal(3600000);
            for (TBDing effectDing : effectDingList) {
                // 有效天数
                actualDays++;
                DingDateTypeEnum dateType = effectDing.getDateType();
                switch (dateType) {
                case WEEKEND:
                    weekends++;
                    break;
                case LEGAL_HOLIDAYS:
                    legalHolidays++;
                    break;
                default:
                    weekDays++;
                    break;
                }
                // 累计工时
                workHours = workHours.add(
                        new BigDecimal(effectDing.getSignOutTime() - effectDing.getSignInTime()).divide(var, 1,
                                RoundingMode.HALF_UP));
            }
            // 违规工时
            int violationDays = Math.toIntExact(
                    dingList.stream().filter(e -> SystemConstant.isOneNull(e.getSignInTime(), e.getSignOutTime()))
                            .count());
            // 考勤异常数
            int dingExceptionCount = 0;
            for (TBDing tbDing : dingList) {
                Long signInTime = tbDing.getSignInTime();
                Long signOutTime = tbDing.getSignOutTime();
                if (!SystemConstant.longNotNull(signInTime)) {
                    dingExceptionCount++;
                }
                if (!SystemConstant.longNotNull(signOutTime)) {
                    dingExceptionCount++;
                }
            }

            // 剩余补卡数
            int remainCount = tbDingService.findRemainCount(userId, serviceId, null);

            // 待处理异常数
            int exceptionCount = tbDingApplyService.count(
                    new QueryWrapper<TBDingApply>().lambda().eq(TBDingApply::getServiceId, serviceId)
                            .eq(TBDingApply::getCreateId, userId).isNull(TBDingApply::getApprove));

            TBDingStatistic tbDingStatistic;
            tbDingStatistic = this.findBySopNoAndUserArchivesId(serviceId, null, userArchivesId);
            if (Objects.isNull(tbDingStatistic)) {
                // 新增考勤统计
                tbDingStatistic = new TBDingStatistic();
                tbDingStatistic.insertInfo(requestUserId);
            } else {
                // 更新考勤统计
                if (!tbDingSubmitService.canUpdateDingSetting(serviceId, null, userId)) {
                    // 已提交的考勤不再计算了
                    continue;
                }
                tbDingStatistic.updateInfo(requestUserId);
            }
            tbDingStatistic.setServiceId(serviceId);
            tbDingStatistic.setServiceName(tbServiceService.getById(serviceId).getName());
            tbDingStatistic.setUserId(userId);
            tbDingStatistic.setUserArchivesId(userArchivesId);
            tbDingStatistic.setUserArchivesName(tbUserArchives.getName());
            tbDingStatistic.setUserArchivesCode(tbUserArchives.getCode());
            tbDingStatistic.setRoleName(sopRoleType);
            tbDingStatistic.setSupplierId(tbUserArchives.getSupplierId());
            tbDingStatistic.setSupplierName(tbUserArchives.getSupplierName());
            tbDingStatistic.setActualDays(actualDays);
            tbDingStatistic.setWeekdays(weekDays);
            tbDingStatistic.setWeekends(weekends);
            tbDingStatistic.setLegalHolidays(legalHolidays);
            tbDingStatistic.setWorkHours(workHours);
            tbDingStatistic.setViolationDays(violationDays);
            tbDingStatistic.setDingExceptionCount(dingExceptionCount);
            tbDingStatistic.setRemainCount(remainCount);
            tbDingStatistic.setExceptionCount(exceptionCount);
            tbDingStatisticList.add(tbDingStatistic);
        }
        this.saveOrUpdateBatch(tbDingStatisticList);
    }

    @Override
    public List<DingStatisticByPersonDto> findStatisticByPerson(Long serviceId) {
        List<DingStatisticByPersonDto> result = new ArrayList<>();
        QueryWrapper<TBDingStatistic> queryWrapper = new QueryWrapper<>();
        if (serviceId != null) {
            queryWrapper.lambda().eq(TBDingStatistic::getServiceId, serviceId);
        }
        List<TBDingStatistic> datasource = this.list(queryWrapper);
        for (TBDingStatistic tbDingStatistic : datasource) {
            DingStatisticByPersonDto dto = new DingStatisticByPersonDto();
            BeanUtils.copyProperties(tbDingStatistic, dto);
            dto.setUserInfo(tbDingStatistic.getUserArchivesName() + "(" + tbDingStatistic.getUserArchivesCode() + ")");
            result.add(dto);
        }
        return result;
    }

    @Override
    public List<DingStatisticBySopDto> findStatisticBySop(Long serviceId) {
        List<DingStatisticBySopDto> result = new ArrayList<>();
        QueryWrapper<TBDingStatistic> queryWrapper = new QueryWrapper<>();
        if (serviceId != null) {
            queryWrapper.lambda().eq(TBDingStatistic::getServiceId, serviceId);
        }
        queryWrapper.lambda().isNotNull(TBDingStatistic::getSopNo);
        List<TBDingStatistic> datasource = this.list(queryWrapper);
        List<String> sopNoList = datasource.stream().map(TBDingStatistic::getSopNo).distinct().collect(Collectors.toList());

        for (String sopNo : sopNoList) {
            TBCrmDetail tbCrmDetail = tbCrmDetailService.findBySopNo(sopNo);
            SopPeriodDto sopPeriodDto = this.findSopPeriod(tbCrmDetail);

            List<TBDingStatistic> tbDingStatisticList = datasource.stream().filter(e -> sopNo.equals(e.getSopNo())).collect(Collectors.toList());
            String serviceName = "";
            int actualDays = 0;
            int weekdays = 0;
            int weekends = 0;
            int legalHolidays = 0;
            BigDecimal workHours = new BigDecimal(0);
            int violationDays = 0;

            for (TBDingStatistic tbDingStatistic : tbDingStatisticList) {
                serviceName = tbDingStatistic.getServiceName();
                actualDays = actualDays + tbDingStatistic.getActualDays();
                weekdays = weekdays + tbDingStatistic.getWeekdays();
                weekends = weekends + tbDingStatistic.getWeekends();
                legalHolidays = legalHolidays + tbDingStatistic.getLegalHolidays();
                workHours = workHours.add(tbDingStatistic.getWorkHours());
                violationDays = violationDays + tbDingStatistic.getViolationDays();
            }
            DingStatisticBySopDto dto = new DingStatisticBySopDto();
            dto.setServiceName(serviceName);
            dto.setSopNo(sopNo);
            dto.setCourseName(tbCrmDetail.getCourseName());
            dto.setActualDays(actualDays);
            dto.setWeekdays(weekdays);
            dto.setWeekends(weekends);
            dto.setLegalHolidays(legalHolidays);
            dto.setWorkHours(workHours.doubleValue());
            dto.setViolationDays(violationDays);
            dto.setScanPeriod(sopPeriodDto.getScanPeriod());
            dto.setMarkPaperPeriod(sopPeriodDto.getMarkPaperPeriod());
            result.add(dto);
        }
        return result;
    }

    @Override
    public List<DingStatisticByCrmDto> findStatisticByCrm(Long serviceId) {
        List<DingStatisticByCrmDto> result = new ArrayList<>();
        QueryWrapper<TBDingStatistic> queryWrapper = new QueryWrapper<>();
        if (serviceId != null) {
            queryWrapper.lambda().eq(TBDingStatistic::getServiceId, serviceId).isNotNull(TBDingStatistic::getSopNo);
        }
        queryWrapper.lambda().isNotNull(TBDingStatistic::getCrmNo);
        List<TBDingStatistic> datasource = this.list(queryWrapper);
        List<String> crmNoList = datasource.stream().map(TBDingStatistic::getCrmNo).distinct().collect(Collectors.toList());

        for (String crmNo : crmNoList) {
            TBCrm tbCrm = tbCrmService.findByCrmNo(crmNo);
            List<TBCrmDetail> tbCrmDetailList = tbCrmDetailService.list(
                    new QueryWrapper<TBCrmDetail>().lambda().eq(TBCrmDetail::getCrmNo, crmNo));
            int scanPeriod = 0;
            int markPaperPeriod = 0;
            for (TBCrmDetail tbCrmDetail : tbCrmDetailList) {
                SopPeriodDto sopPeriodDto = this.findSopPeriod(tbCrmDetail);
                scanPeriod = scanPeriod + sopPeriodDto.getScanPeriod();
                markPaperPeriod = markPaperPeriod + sopPeriodDto.getMarkPaperPeriod();
            }

            List<TBDingStatistic> tbDingStatisticList = datasource.stream().filter(e -> crmNo.equals(e.getCrmNo())).collect(Collectors.toList());
            String serviceName = "";
            int actualDays = 0;
            int weekdays = 0;
            int weekends = 0;
            int legalHolidays = 0;
            BigDecimal workHours = new BigDecimal(0);
            int violationDays = 0;

            for (TBDingStatistic tbDingStatistic : tbDingStatisticList) {
                serviceName = tbDingStatistic.getServiceName();
                actualDays = actualDays + tbDingStatistic.getActualDays();
                weekdays = weekdays + tbDingStatistic.getWeekdays();
                weekends = weekends + tbDingStatistic.getWeekends();
                legalHolidays = legalHolidays + tbDingStatistic.getLegalHolidays();
                workHours = workHours.add(tbDingStatistic.getWorkHours());
                violationDays = violationDays + tbDingStatistic.getViolationDays();
            }
            DingStatisticByCrmDto dto = new DingStatisticByCrmDto();
            dto.setServiceName(serviceName);
            dto.setCrmNo(crmNo);
            dto.setCrmName(tbCrm.getName());
            dto.setActualDays(actualDays);
            dto.setWeekdays(weekdays);
            dto.setWeekends(weekends);
            dto.setLegalHolidays(legalHolidays);
            dto.setWorkHours(workHours.doubleValue());
            dto.setViolationDays(violationDays);
            dto.setScanPeriod(scanPeriod);
            dto.setMarkPaperPeriod(markPaperPeriod);
            result.add(dto);
        }
        return result;
    }

    @Override
    public List<DingStatisticByCustomDto> findStatisticByCustom(Long serviceId) {
        List<DingStatisticByCustomDto> result = new ArrayList<>();
        QueryWrapper<TBDingStatistic> queryWrapper = new QueryWrapper<>();
        if (serviceId != null) {
            queryWrapper.lambda().eq(TBDingStatistic::getServiceId, serviceId);
        }
        queryWrapper.lambda().isNotNull(TBDingStatistic::getSopNo);
        List<TBDingStatistic> datasource = this.list(queryWrapper);
        List<Long> customIdList = datasource.stream().map(TBDingStatistic::getCustomId).distinct().collect(Collectors.toList());

        for (Long customId : customIdList) {
            List<TBCrmDetail> tbCrmDetailList = tbCrmDetailService.findByServiceIdAndCustomId(serviceId, customId);
            int scanPeriod = 0;
            int markPaperPeriod = 0;
            for (TBCrmDetail tbCrmDetail : tbCrmDetailList) {
                SopPeriodDto sopPeriodDto = this.findSopPeriod(tbCrmDetail);
                scanPeriod = scanPeriod + sopPeriodDto.getScanPeriod();
                markPaperPeriod = markPaperPeriod + sopPeriodDto.getMarkPaperPeriod();
            }

            List<TBDingStatistic> tbDingStatisticList = datasource.stream().filter(e -> customId.equals(e.getCustomId())).collect(Collectors.toList());
            String serviceName = "";
            String customName = sysCustomService.getById(customId).getName();
            int actualDays = 0;
            int weekdays = 0;
            int weekends = 0;
            int legalHolidays = 0;
            BigDecimal workHours = new BigDecimal(0);
            int violationDays = 0;

            for (TBDingStatistic tbDingStatistic : tbDingStatisticList) {
                serviceName = tbDingStatistic.getServiceName();
                actualDays = actualDays + tbDingStatistic.getActualDays();
                weekdays = weekdays + tbDingStatistic.getWeekdays();
                weekends = weekends + tbDingStatistic.getWeekends();
                legalHolidays = legalHolidays + tbDingStatistic.getLegalHolidays();
                workHours = workHours.add(tbDingStatistic.getWorkHours());
                violationDays = violationDays + tbDingStatistic.getViolationDays();
            }
            DingStatisticByCustomDto dto = new DingStatisticByCustomDto();
            dto.setServiceName(serviceName);
            dto.setCustomName(customName);
            dto.setActualDays(actualDays);
            dto.setWeekdays(weekdays);
            dto.setWeekends(weekends);
            dto.setLegalHolidays(legalHolidays);
            dto.setWorkHours(workHours.doubleValue());
            dto.setViolationDays(violationDays);
            dto.setScanPeriod(scanPeriod);
            dto.setMarkPaperPeriod(markPaperPeriod);
            result.add(dto);
        }
        return result;
    }

    @Override
    public List<DingStatisticBySupplierDto> findStatisticBySupplier(Long serviceId) {
        List<DingStatisticBySupplierDto> result = new ArrayList<>();
        QueryWrapper<TBDingStatistic> queryWrapper = new QueryWrapper<>();
        if (serviceId != null) {
            queryWrapper.lambda().eq(TBDingStatistic::getServiceId, serviceId);
        }
        List<TBDingStatistic> datasource = this.list(queryWrapper);
        List<Long> supplierIdList = datasource.stream().map(TBDingStatistic::getSupplierId).distinct().collect(Collectors.toList());

        for (Long supplierId : supplierIdList) {
            List<TBDingStatistic> tbDingStatisticList = datasource.stream().filter(e -> supplierId.equals(e.getSupplierId())).collect(Collectors.toList());
            String serviceName = "";
            String supplierName = sysSupplierService.getById(supplierId).getName();
            int actualDays = 0;
            int weekdays = 0;
            int weekends = 0;
            int legalHolidays = 0;
            BigDecimal workHours = new BigDecimal(0);
            int violationDays = 0;

            for (TBDingStatistic tbDingStatistic : tbDingStatisticList) {
                serviceName = tbDingStatistic.getServiceName();
                actualDays = actualDays + tbDingStatistic.getActualDays();
                weekdays = weekdays + tbDingStatistic.getWeekdays();
                weekends = weekends + tbDingStatistic.getWeekends();
                legalHolidays = legalHolidays + tbDingStatistic.getLegalHolidays();
                workHours = workHours.add(tbDingStatistic.getWorkHours());
                violationDays = violationDays + tbDingStatistic.getViolationDays();
            }
            DingStatisticBySupplierDto dto = new DingStatisticBySupplierDto();
            dto.setServiceName(serviceName);
            dto.setSupplierName(supplierName);
            dto.setActualDays(actualDays);
            dto.setWeekdays(weekdays);
            dto.setWeekends(weekends);
            dto.setLegalHolidays(legalHolidays);
            dto.setWorkHours(workHours.doubleValue());
            dto.setViolationDays(violationDays);
            result.add(dto);
        }
        return result;
    }

    /**
     * 查询sop周期
     *
     * @param tbCrmDetail 派单详情
     * @return 结果
     */
    private SopPeriodDto findSopPeriod(TBCrmDetail tbCrmDetail) {
        Long scanActualStartTime = tbCrmDetail.getScanActualStartTime();
        Long scanActualEndTime = tbCrmDetail.getScanActualEndTime();
        Long markPaperActualStartTime = tbCrmDetail.getMarkPaperActualStartTime();
        Long markPaperActualEndTime = tbCrmDetail.getMarkPaperActualEndTime();
        int scanPeriod = 0;
        int markPaperPeriod = 0;
        if (SystemConstant.longNotNull(scanActualStartTime) && SystemConstant.longNotNull(scanActualEndTime)) {
            scanPeriod = DateDisposeUtils.getDaysBetween(scanActualStartTime, scanActualEndTime, SystemConstant.DEFAULT_DATE_YMD_S_PATTERN).size();
        }
        if (SystemConstant.longNotNull(markPaperActualStartTime) && SystemConstant.longNotNull(
                markPaperActualEndTime)) {
            markPaperPeriod = DateDisposeUtils.getDaysBetween(markPaperActualStartTime, markPaperActualEndTime,
                    SystemConstant.DEFAULT_DATE_YMD_S_PATTERN).size();
        }
        SopPeriodDto sopPeriodDto = new SopPeriodDto();
        sopPeriodDto.setScanPeriod(scanPeriod);
        sopPeriodDto.setMarkPaperPeriod(markPaperPeriod);
        return sopPeriodDto;
    }
}
